package uulm.teamname.marvelous.server.lobby.pipelining;

import org.tinylog.Logger;
import uulm.teamname.marvelous.gamelibrary.events.Event;
import uulm.teamname.marvelous.gamelibrary.events.EventBuilder;
import uulm.teamname.marvelous.gamelibrary.gamelogic.GameInstance;
import uulm.teamname.marvelous.gamelibrary.requests.Request;
import uulm.teamname.marvelous.gamelibrary.requests.RequestType;
import uulm.teamname.marvelous.server.lobby.Lobby;

import java.util.List;
import java.util.concurrent.atomic.AtomicBoolean;

public class GameStateSegment implements Segment {
    private GameInstance game;

    public GameStateSegment(GameInstance game){
        this.game = game;
    }

    @Override
    public void processRequests(Packet packet, List<Event> carrier, AtomicBoolean abort) {
        Logger.trace("GameStateSegment received {} requests.", packet.size());
        var result = game.checkRequestsAndApply(packet);
        Logger.trace("GameLogic generated {} events", result.map(List::size).orElse(0));
        if (result.isPresent()) {
            Logger.trace("Result from GameLogic is present. Adding requests to carrier.");
            carrier.addAll(result.get());
            packet.clear();
        } else {
            Logger.debug("Result from GameLogic is invalid. Triggering error.");
            abort.set(true);
        }
    }
}
